@@ -16,6 +16,7 @@
 #  define SEQ_NONE      seq_inline
 #  define MAP_NONE      map_inline
 #  define TYPE_IS_NULL(x) ((x == NULL) || (strcmp( x, "str" ) == 0))
+#  define OBJOF(a)        (a)
 #else
 #  define PACKAGE_NAME  "YAML::Syck"
 #  define NULL_LITERAL  "~"
@@ -25,6 +26,7 @@
 #  define SEQ_NONE      seq_none
 #  define MAP_NONE      map_none
 #  define TYPE_IS_NULL(x) (x == NULL)
+#  define OBJOF(a)        (*tag ? tag : a)
 #endif
 
 /*
@@ -220,6 +222,37 @@ static char* perl_json_preprocess(char *s) {
     return out;
 }
 
+void perl_json_postprocess(SV *sv) {
+    int i;
+    char ch;
+    bool in_string = 0;
+    bool in_quote  = 0;
+    char *pos;
+    char *s = SvPVX(sv);
+    STRLEN len = sv_len(sv);
+    STRLEN final_len = len;
+
+    pos = s;
+
+    for (i = 0; i < len; i++) {
+        ch = *(s+i);
+        *pos++ = ch;
+        if (in_quote) {
+            in_quote = !in_quote;
+        }
+        else if (ch == '\"') {
+            in_string = !in_string;
+        }
+        else if ((ch == ':' || ch == ',') && !in_string) {
+            i++; /* has to be a space afterwards */
+            final_len--;
+        }
+    }
+    *pos = '\0';
+
+    SvCUR_set(sv, final_len);
+}
+
 static SV * Load(char *s) {
     SYMID v;
     SyckParser *parser;
@@ -268,6 +301,7 @@ void perl_syck_emitter_handler(SyckEmitter *e, st_data_t data) {
         mg_get(sv);
     }
 
+#ifndef YAML_IS_JSON
     if (sv_isobject(sv)) {
         ref = savepv(sv_reftype(SvRV(sv), TRUE));
         *tag = '\0';
@@ -280,6 +314,7 @@ void perl_syck_emitter_handler(SyckEmitter *e, st_data_t data) {
         }
         strcat(tag, ref);
     }
+#endif
 
     if (SvROK(sv)) {
         perl_syck_emitter_handler(e, (st_data_t)SvRV(sv));
@@ -287,14 +322,12 @@ void perl_syck_emitter_handler(SyckEmitter *e, st_data_t data) {
         return;
     }
 
-#define OBJOF(a) (*tag ? tag : a)
     switch (SvTYPE(sv)) {
         case SVt_NULL: { return; }
-        case SVt_PV:
         case SVt_PVIV:
         case SVt_PVNV: {
-            if (SvCUR(sv) > 0) {
-                syck_emit_scalar(e, OBJOF("string"), SCALAR_STRING, 0, 0, 0, SvPVX(sv), SvCUR(sv));
+            if (sv_len(sv) > 0) {
+                syck_emit_scalar(e, OBJOF("string"), SvNIOK(sv) ? SCALAR_NUMBER : SCALAR_STRING, 0, 0, 0, SvPV_nolen(sv), sv_len(sv));
             }
             else {
                 syck_emit_scalar(e, OBJOF("string"), SCALAR_QUOTED, 0, 0, 0, "", 0);
@@ -302,12 +335,21 @@ void perl_syck_emitter_handler(SyckEmitter *e, st_data_t data) {
             break;
         }
         case SVt_IV:
-        case SVt_NV:
+        case SVt_NV: {
+            if (sv_len(sv) > 0) {
+                syck_emit_scalar(e, OBJOF("string"), SCALAR_NUMBER, 0, 0, 0, SvPV_nolen(sv), sv_len(sv));
+            }
+            else {
+                syck_emit_scalar(e, OBJOF("string"), SCALAR_QUOTED, 0, 0, 0, "", 0);
+            }
+            break;
+        }
+        case SVt_PV:
         case SVt_PVMG:
         case SVt_PVBM:
         case SVt_PVLV: {
             if (sv_len(sv) > 0) {
-                syck_emit_scalar(e, OBJOF("string"), SCALAR_NUMBER, 0, 0, 0, SvPV_nolen(sv), sv_len(sv));
+                syck_emit_scalar(e, OBJOF("string"), SCALAR_STRING, 0, 0, 0, SvPV_nolen(sv), sv_len(sv));
             }
             else {
                 syck_emit_scalar(e, OBJOF("string"), SCALAR_QUOTED, 0, 0, 0, "", 0);
@@ -402,5 +444,9 @@ SV* Dump(SV *sv) {
     }
 #endif
 
+#ifdef YAML_IS_JSON
+    perl_json_postprocess(out);
+#endif
+
     return out;
 }
